% Complete rewrite of the test_gpca function to use the generalized data
% generation function instead.

addpath helper_functions

clear;
close all;
clc;

roundNumber = 1;

seedTimeStamp = sum(100*clock); %% Generate a new random seed.
rand('state',seedTimeStamp);
randn('state', seedTimeStamp);


% Generate the data samples
ambientDimension = 3;
subspaceDimensions = [2 1];
groupCount = length(subspaceDimensions);
noiseLevel = 0.06;
outlierPercentage = 0;

basisErrorStat = 0;
segmentationErrorStat = 0;
for roundIndex=1:roundNumber
    [X, aprioriSampleLabels, aprioriGroupBases] = generate_samples('ambientSpaceDimension', ambientDimension,...
        'groupSizes',[100 400],...
        'basisDimensions', subspaceDimensions,...
        'noiseLevel', noiseLevel,...
        'outlierPercentage',outlierPercentage,...
        'minimumSubspaceAngle', pi/4);
    sampleNumber = size(X,2);
        
    % Perform Ksubspaces on the data
    [sampleLabels, groupBases] = rgpca_leave_one_out(X, subspaceDimensions,'outlierPercentage', 0.2,...
        'postoptimization',true,'angleTolerance',0.4,'nullspacemethod',1);


    % Reorder the bases and sample labels so that they are likely to agree with
    % the original ones.
    [bestMapping, errorProbability] = relabel_samples(aprioriSampleLabels, sampleLabels, subspaceDimensions);
    segmentationErrorStat = segmentationErrorStat + errorProbability;
    inverseMapping(bestMapping) = 1:length(bestMapping);
    
    fixedSampleLabels = -ones(1,sampleNumber);
    inlierIndex=find(sampleLabels~=-1);
    fixedSampleLabels(inlierIndex) = bestMapping(sampleLabels(inlierIndex));
    for subspaceIndex=1:groupCount
        fixedBases{subspaceIndex} = groupBases{inverseMapping(subspaceIndex)};
    end
    basisErrorStat = basisErrorStat + average_basis_error(aprioriGroupBases, fixedBases);
    
    % Display up-to-date error statistics averaged over roundIndex
    disp(['Try #' num2str(roundIndex) '. Segmentation Error: ' num2str(segmentationErrorStat/roundIndex*100) '%.  Average Basis Error: ' num2str(basisErrorStat/roundIndex)]);
end

% Plot the gpca results
if ambientDimension<=3
    figureNumber = figure();
    subplot(1,2,1);
    plot_data(X, aprioriSampleLabels);
    hold on;
    plot_subspaces(X, aprioriSampleLabels, aprioriGroupBases, subspaceDimensions);
    hold off;
    title('Apriori Data');
    set(0,'CurrentFigure',figureNumber); % Change back to the correct figure without changing its visibility.
    subplot(1,2,2);
    plot_data(X, fixedSampleLabels);
    hold on;
    plot_subspaces(X, fixedSampleLabels, fixedBases, subspaceDimensions);
    hold off;
    title('Final Segmentation')
end

